/*
 * deltas_c.h
 *
 * Calculate deltas on the fly
 * for sequential output from rasta to pfile
 * 1997jun18 dpwe@icsi.berkeley.edu
 * $Header: /u/drspeech/src/rasta/RCS/deltas_c.h,v 1.1 1997/06/27 01:02:21 dpwe Exp $
 */

/* #include "rasta.h"	/* definition of struct fvec etc */

typedef struct _DeltaCalc {
    /* Class that acts as a 'filter' to append deltas to features 
       it receives */
    struct fmat *state;	/* history of featues, for calculating deltas */
    struct fvec *dfilt;	/* FIR filter for deltas */
    struct fvec *ddfilt; /* FIR filter for double-deltas */
    int rownum;		/* how many rows we've been given */
    int minrows;	/* we need this many rows before we emit any */
    int degree;		/* 0 = pass-thru, 1 = deltas, 2 = deltas+ddeltas */
    int nfeas;		/* feature count */
    int firstpad;	/* how many copies of the first frame to use */
    int middle;		/* which row is the middle? */
    int dfoffset;	/* apply dfilt starting at this row of state */
    int unpadding;	/* count how many post-end frames we've returned */

    int nOutputFtrs;	/* output columns */
    struct fvec *retvec;	/* structure for returning result */
} DeltaCalc;

DeltaCalc* alloc_DeltaCalc(int feas, int winlen, int dgree);
    /* Initialize the object that will "filter" feature vectors 
       to return them augmented by their deltas. 
       <feas> is the size of the vectors we will be passed; <winlen> 
       is the window length over which deltas are to be calculated, 
       and <dgree> is 1 for plain deltas, or 2 for deltas and 
       double deltas. */

void free_DeltaCalc(DeltaCalc *dc);
    /* free the DeltaCalc object */

void DeltaCalc_reset(DeltaCalc *dc);
    /* return to initial state */

struct fvec *DeltaCalc_appendDeltas(DeltaCalc *dc, const struct fvec *infeas);
    /* Take a row of features, and append it to our state.  When 
       we have enough state, return a nonempty floatRef with 
       fully-padded feature rows for output. 
       When caller reaches end of input, they repeatedly call DeltaCalc 
       with a zero-len <infeas> to allow it to flush its state; DeltaCalc
       will eventually give a zero-len response, meaning all done. */
